const XHTML_NS = "http://www.w3.org/1999/xhtml";

var compositeOpenWindows = new Array();
var isMac = (navigator.platform.toLowerCase().indexOf("mac") != -1);

/* trap the close event to make sure the user doesn't close and lose data */
window.addEventListener("close", checkOpen, false);

/*
 * register event listeners for textareas
 */
function compositeInit() {
  setCompositeDefaultPrefs();
  var myDoc = getBrowser().contentDocument;
  var textareas = myDoc.getElementsByTagName("textarea");

  if (textareas.length == 0)
    return;

  var use_keypress = getPref().GetBoolPref(PREF_USE_KEYPRESS);
  var use_button = getPref().GetBoolPref(PREF_USE_BUTTON);
  for (var i = 0; i < textareas.length; i++) {
    if (use_keypress)
      textareas[i].addEventListener("keypress", spawnEditor, false);
    if (use_button)
      addCompositeButton(textareas[i]);
  }
}

/*
 * Add an edit button for the document
 */
function addCompositeButton(textarea) {
  var divWrapperClassName = "-composite-textarea-wrapper-div";

  if (textarea.parentNode.className == divWrapperClassName)
    return; /* avoid wrapping twice (I hope) */

  var lambda = function() {
    openWindowForTextarea(textarea);
  }

  textarea.display = "block";

  var wrapper = document.createElementNS(XHTML_NS, "div");
  wrapper.className = divWrapperClassName;
  wrapper.style.display = "table-cell";

  var editButton = document.createElementNS(XHTML_NS, "input");
  editButton.type = "button"
  editButton.value = "Edit with Composite";
  editButton.addEventListener("click", lambda, false);

  textarea.parentNode.replaceChild(wrapper, textarea);
  wrapper.appendChild(textarea);
  wrapper.appendChild(editButton);
}

/*
 * spawn a new composite instance for a textarea.
 */
function spawnEditor(e) {
  if ((!isMac && e.ctrlKey && e.which == 101) || /* ctrl-e for non-mac */
    (isMac && e.metaKey && e.which == 101)) { /* meta (cmd)-e for mac */
    /* XXX the isMac modifier may not be right, xbl would be much easier */
    e.preventDefault();
    openWindowForTextarea(e.currentTarget);
  }
}

/*
 * open a new composite instance for a textarea.  If it already is open,
 * just focus() it.
 */
function openWindowForTextarea(textarea) {
  for(var i = 0; i < compositeOpenWindows.length; i++) {
    var windowPair = compositeOpenWindows[i];
    if (windowPair.textarea == textarea) {
      windowPair.window.focus();
      return;
    }
  }

  var windowPair = new Array();
  windowPair.textarea = textarea;
  windowPair.window = window.openDialog('chrome://composite/content/editor.xul',
    '_blank', 'chrome,all,dialog=no', textarea);
  textarea.addEventListener("dblclick", focusCompositeWindow, false);
  if (getPref().GetBoolPref(PREF_DISABLE_TEXTAREA))
    windowPair.textarea.disabled = true;
  compositeOpenWindows[compositeOpenWindows.length] = windowPair;
}

/*
 * remove the window/textarea pair from the list of open composite instances
 */
function closeWindowForTextarea(textarea) {
  for (var i = 0; i < compositeOpenWindows.length; i++) {
    var windowPair = compositeOpenWindows[i];
    if (windowPair.textarea == textarea) {
      compositeOpenWindows.splice(i, 1);

      if (getPref().GetBoolPref(PREF_DISABLE_TEXTAREA))
        windowPair.textarea.disabled = false;

      windowPair.window.close();
      windowPair.textarea.removeEventListener("dblclick", focusCompositeWindow, false);
      return;
    }
  }
}

/*
 * Used by the event listener to focus composite on double-click
 */
function focusCompositeWindow(e) {
  for (var i = 0; i < compositeOpenWindows.length; i++) {
    var windowPair = compositeOpenWindows[i];
    if (windowPair.textarea == e.currentTarget) {
      windowPair.window.focus();
      return;
    }
  }
}

/*
 * Prevent user dataloss by 'accidental' close
 */
function checkOpen(e) {
  if (compositeOpenWindows.length > 0) {
    if (confirm("There are still open Composite editors, do you wish to close and lose their contents?")) {
      for (var i = compositeOpenWindows.length - 1; i >= 0; i--) {
        var windowPair = compositeOpenWindows[i];
        if (windowPair.window)
          windowPair.window.close();
      }
    } else { /* user hit cancel, cancel window close event */
      e.preventDefault();

      return false;
    }
  }

  return true;
}
